#
# Copyright (c) 2008 CNRS/LAAS
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# Matthieu Herrb - Fri Nov 14 2008
#

package require tdom

proc readDoc { file } {
    if {[catch {set f [open $file r]} msg] != 0} {
	error "open: $msg"
    }
    while {![eof $f]} {
	append xml [gets $f]
    }
    close $f
    set dom [dom parse $xml]
    return [$dom documentElement]
}

if { $argc != 2 } {
    error "Usage: gen_calib VELODYNE_32|VELODYNE_64 <file>"
}

set file [lindex $argv 1]
if {[catch {readDoc $file } doc] != 0} {
    puts "gen_calib: $doc"
    exit 2
}

set model [lindex $argv 0]
switch $model {
	VELODYNE_32 {
		set modelNumber 32
	}
	VELODYNE_64 {
		set modelNumber 64
	}
	default {
		error "Usage: gen_calib VELODYNE_32|VELODYNE_64 <file>"
	}
}

puts "/* Velodyne HDL-$modelNumber calibration data generated from $file */"
puts "/* Do not edit manually */"
puts ""

set n [[$doc selectNodes //DB/points_/count ] text]

puts "const VELODYNE_CALIBRATION vc$modelNumber\[$n\] = {"

set points [$doc selectNodes "//DB/points_/item/px"]
foreach p $points {
    set rotCorrection [[$p getElementsByTagName rotCorrection_] text]
    set vertCorrection [[$p getElementsByTagName vertCorrection_] text]
    set distCorrection [[$p getElementsByTagName distCorrection_] text]
    set vertOffset [[$p getElementsByTagName vertOffsetCorrection_] text] 
    set horizOffset [[$p getElementsByTagName horizOffsetCorrection_] text]

    puts -nonewline "\t{ $rotCorrection, $vertCorrection, $distCorrection, "
    puts  "$vertOffset, $horizOffset },"
}
puts "};"

