/*
 * Copyright (c) 2008,2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "../src/velodyneConfig.h"

#include <sys/time.h>	/* struct timeval */
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <velodyne/velodyneLib.h>

int
main(int argc, char *argv[])
{
	char *modelArg;
	char *name;
	VELODYNE_INTERFACE *dev;
	VELODYNE_MODEL model;
	VELODYNE_PACKET packet;
	VELODYNE_CARTESIAN_PACKET cartPacket;
	char errMsg[]
		= "Error. Usage: velodyneAcquire VELODYNE_32|VELODYNE_64 [<file>].";

	if (argc == 3) {
		modelArg = argv[1];
		if (strcmp(modelArg, "VELODYNE_32") == 0) {
			model = VELODYNE_32;
		} else if (strcmp(modelArg, "VELODYNE_64") == 0) {
			model = VELODYNE_64;
		} else {
			fprintf(stderr, "%s\n", errMsg);
			exit (2);
		}
		name = argv[2];
		dev = velodyne_open_file(model, name);
	} else if (argc == 2) {
		modelArg = argv[1];
		if (strcmp(modelArg, "VELODYNE_32") == 0) {
			model = VELODYNE_32;
		} else if (strcmp(modelArg, "VELODYNE_64") == 0) {
			model = VELODYNE_64;
		} else {
			fprintf(stderr, "%s\n", errMsg);
			exit (2);
		}
		dev = velodyne_open_network(model);
	} else {
		fprintf(stderr, "%s\n", errMsg);
		exit (2);
	}

	if (dev == NULL) {
#ifdef HAVE_POCOLIBS
		h2perror("velodyne_open");
#else
		perror("velodyne_open");
#endif
		exit (2);
	}
	while (1) {
		if (velodyne_read_packet(dev, &packet) != VELODYNE_PAYLOAD_SIZE)
			break;
		velodyne_print_packet(&packet);
		velodyne_to_cartesian_packet(dev->model, &packet, &cartPacket);
		velodyne_print_cartesian_packet(&cartPacket);
	}
	velodyne_close(dev);

	exit(0);
}
